﻿# Đường dẫn tới file CSV 
$filePath = "C:\AES\test.csv"
$keyFilePath = "C:\AES\encryptionKey.bin"

# Hàm giải mã một chuỗi bằng AES
function Decrypt-String {
    param (
        [string]$cipherTextWithIV,
        [byte[]]$key
    )

    # Tách IV và Ciphertext
    $parts = $cipherTextWithIV -split ":"
    
    if ($parts.Length -ne 2) {
        throw "Dữ liệu giải mã không hợp lệ. Vui lòng kiểm tra định dạng IV:Ciphertext."
    }

    $iv = [Convert]::FromBase64String($parts[0])
    $cipherText = [Convert]::FromBase64String($parts[1])

    $aes = [System.Security.Cryptography.Aes]::Create()
    $aes.Key = $key
    $aes.IV = $iv

    $decryptor = $aes.CreateDecryptor($aes.Key, $aes.IV)

    try {
        $ms = New-Object System.IO.MemoryStream
        $ms.Write($cipherText, 0, $cipherText.Length)
        $ms.Position = 0
        $cs = New-Object System.Security.Cryptography.CryptoStream($ms, $decryptor, [System.Security.Cryptography.CryptoStreamMode]::Read)
        $reader = New-Object System.IO.StreamReader($cs)

        return $reader.ReadToEnd()
    } catch {
        throw "Lỗi khi giải mã: $_"
    } finally {
        $reader.Close()
        $cs.Close()
        $ms.Close()
    }
}

# Hàm tạo password ngẫu nhiên
function Generate-RandomPassword {
    $length = 12  # Độ dài password
    $chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*()"
    $random = New-Object System.Random
    return -join ((0..($length - 1)) | ForEach-Object { $chars[$random.Next(0, $chars.Length)] })
}

# Hàm mã hóa một chuỗi bằng AES
function Encrypt-String {
    param (
        [string]$plainText,
        [byte[]]$key
    )

    $aes = [System.Security.Cryptography.Aes]::Create()
    $aes.Key = $key
    $aes.GenerateIV()
    $iv = $aes.IV

    $encryptor = $aes.CreateEncryptor($aes.Key, $iv)
    $ms = New-Object System.IO.MemoryStream
    $cs = New-Object System.Security.Cryptography.CryptoStream($ms, $encryptor, [System.Security.Cryptography.CryptoStreamMode]::Write)

    $writer = New-Object System.IO.StreamWriter($cs)
    $writer.Write($plainText)
    $writer.Close()
    $cs.Close()

    # Trả về định dạng "IV:Ciphertext"
    return [Convert]::ToBase64String($iv) + ":" + [Convert]::ToBase64String($ms.ToArray())
}

# Kiểm tra và tạo khóa nếu file khóa không tồn tại
if (-Not (Test-Path $keyFilePath)) {
    $key = New-Object byte[] 32   # 32 bytes cho AES-256
    (New-Object System.Security.Cryptography.RNGCryptoServiceProvider).GetBytes($key)
    [System.IO.File]::WriteAllBytes($keyFilePath, $key)
    Write-Host "Đã tạo khóa mã hóa mới và lưu vào file."
} else {
    $key = [System.IO.File]::ReadAllBytes($keyFilePath)
}

# Đọc nội dung file CSV
$data = Import-Csv -Path $filePath

# Giải mã password, reset password, và mã hóa lại password mới
foreach ($row in $data) {
    $ip = $row.IP
    $username = $row.Username
    $password = $row.Password

    try {
        $decryptedPassword = $null
        # Giải mã password
        try {
            $decryptedPassword = Decrypt-String -cipherTextWithIV $password -key $key
            Write-Host "Password đã được giải mã cho $username trên máy $ip $decryptedPassword"
        } catch {
            # Nếu giải mã thất bại, giả định password là plaintext
            $decryptedPassword = $password
            Write-Host "Password cho $username trên máy $ip là plaintext: $decryptedPassword"
        }

        # Tạo password mới ngẫu nhiên
        $newPasswordPlainText = Generate-RandomPassword

        # Tạo đối tượng thông tin xác thực
        $oldPassword = ConvertTo-SecureString $decryptedPassword -AsPlainText -Force
        $credential = New-Object System.Management.Automation.PSCredential($username, $oldPassword)

        # Kết nối từ xa và đổi password
        $session = New-PSSession -ComputerName $ip -Credential $credential

        # Đổi password cho user từ xa
        Invoke-Command -Session $session -ScriptBlock {
            param ($newPassword)
            net user $using:username $newPassword
        } -ArgumentList $newPasswordPlainText

        # Mã hóa password mới
        $row.Password = Encrypt-String -plainText $newPasswordPlainText -key $key

        # Đóng session từ xa
        Remove-PSSession -Session $session

        Write-Host "Đã đổi password cho $username trên máy $ip thành công. Password mới: $newPasswordPlainText"

    } catch {
        Write-Host "Có lỗi khi kết nối hoặc đổi password cho $username trên máy $ip $_"
    }
}

# Ghi đè lại file CSV với password đã được mã hóa
$data | Export-Csv -Path $filePath -NoTypeInformation

Write-Host "Password mới đã được mã hóa và ghi đè lên file CSV."
